
(function ($) {

/**
 * Modal Frame object for child windows.
 */
Drupal.modalFrameChild = Drupal.modalFrameChild || {
  processed: false,
  behaviors: {}
};

/**
 * Child dialog behavior.
 */
Drupal.modalFrameChild.attach = function(context) {

  var self = Drupal.modalFrameChild;
  var settings = Drupal.settings.modalFrameChild || {};

  // If we cannot reach the parent window, then we have nothing else todo here.
  if (!self.isObject(parent.Drupal) || !self.isObject(parent.Drupal.modalFrame)) {
    return;
  }

  // Shortcuts to parent objects.
  self.$parentWindow = parent.jQuery(parent);
  self.parentModalFrame = parent.Drupal.modalFrame;

  // Make sure this behavior is not processed more than once.
  if (!self.processed) {
    self.processed = true;
    self.parentModalFrame.iframe.$container.addClass('modalframe-loaded');
    $('.ui-dialog-title', self.parentModalFrame.iframe.$container.prev()).html($(document).attr('title'));
  }
  
  // Attach child related behaviors to the current context.
  self.attachBehaviors(context);
  
};

/**
 * Trigger a custom event on the parent window.
 */
Drupal.modalFrameChild.triggerParentEvent = function(eventName, eventArguments) {
  this.$parentWindow.trigger(this.parentModalFrame.eventHandlerName(eventName), eventArguments);
};

/**
 * Check if the given variable is an object.
 */
Drupal.modalFrameChild.isObject = function(something) {
  return (something !== null && typeof something === 'object');
};

/**
 * Attach child related behaviors to the iframed document.
 */
Drupal.modalFrameChild.attachBehaviors = function(context) {
  $.each(this.behaviors, function() {
    this(context);
  });
};

/**
 * Attach our own behavior on top of the list of existing behaviors.
 *
 * We need to execute before everything else because the child frame is not
 * visible until we bind the child window to the parent, and this may cause
 * problems with other behaviors that need the document visible in order to
 * do its own job.
 */
Drupal.behaviors = $.extend({modalFrameChild: Drupal.modalFrameChild.attach}, Drupal.behaviors);

})(jQuery);
